/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
/*
 * Copyright 2009 Lukasz Wozniak
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 * http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
package com.sos.VirtualFileSystem.CredentialStore.KeePass.pl.sind.keepass.kdb.v1;
import java.nio.ByteBuffer;
import java.util.Arrays;

import org.apache.log4j.Logger;

public class Field {
	@SuppressWarnings("unused") private final String		conClassName		= this.getClass().getSimpleName();
	@SuppressWarnings("unused") private static final String	conSVNVersion		= "$Id$";
	@SuppressWarnings("unused") private final Logger		logger				= Logger.getLogger(this.getClass());
	public static final int									DATE_FIELD_SIZE		= 5;
	public static final int									ID_FIELD_SIZE		= 4;
	public static final int									FLAGS_FIELD_SIZE	= 4;
	public static final int									LEVEL_FIELD_SIZE	= 2;
	public static final int									UUID_FIELD_SIZE		= 16;
	private short											fieldType;
	private int												fieldSize;
	private byte[]											fieldData;

	public Field(final short fieldType, final int fieldSize, final int expectedFieldSize, final ByteBuffer data) {
		super();
		if (fieldSize != expectedFieldSize) {
			throw new IllegalArgumentException(String.format("Invalid field size for %s. Expecting %d found %d.", getClass().getSimpleName(),
					expectedFieldSize, fieldSize));
		}
		this.fieldType = fieldType;
		this.fieldSize = fieldSize;
		fieldData = new byte[fieldSize];
		data.get(fieldData);
	}

	public short getFieldType() {
		return fieldType;
	}

	public void setFieldType(final short fieldType) {
		this.fieldType = fieldType;
	}

	public int getFieldSize() {
		return fieldSize;
	}

	public void setFieldSize(final int fieldSize) {
		this.fieldSize = fieldSize;
	}

	public byte[] getFieldData() {
		return fieldData;
	}

	public void setFieldData(final byte[] fieldData) {
		this.fieldData = fieldData;
	}

	@Override public String toString() {
		return "Field [fieldData=" + Arrays.toString(fieldData) + ", fieldSize=" + fieldSize + ", fieldType=" + fieldType + "]";
	}

	@Override public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + Arrays.hashCode(fieldData);
		result = prime * result + fieldSize;
		result = prime * result + fieldType;
		return result;
	}

	@Override public boolean equals(final Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Field other = (Field) obj;
		if (!Arrays.equals(fieldData, other.fieldData))
			return false;
		if (fieldSize != other.fieldSize)
			return false;
		if (fieldType != other.fieldType)
			return false;
		return true;
	}
}
