/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
/**
 *
 */
package com.sos.VirtualFileSystem.DataElements;
import org.apache.log4j.Logger;

import com.sos.JSHelper.Exceptions.JobSchedulerException;
import com.sos.VirtualFileSystem.Factory.VFSFactory;
import com.sos.VirtualFileSystem.Interfaces.ISOSVFSHandler;
import com.sos.VirtualFileSystem.Interfaces.ISOSVfsFileTransfer;
import com.sos.VirtualFileSystem.Options.SOSConnection2OptionsAlternate;
import com.sos.VirtualFileSystem.Options.SOSFTPOptions;

/**
 * @author KB
 *
 */
public class SOSVfsConnectionFactory {
	@SuppressWarnings("unused") private final String		conClassName		= this.getClass().getSimpleName();
	@SuppressWarnings("unused") private static final String	conSVNVersion		= "$Id: SOSVfsConnectionFactory.java 23649 2014-04-09 22:38:14Z kb $";
	private final static Logger		logger				= Logger.getLogger(SOSVfsConnectionFactory.class);
	protected SOSFTPOptions									objOptions			= null;
	private SOSVfsConnectionPool							objConnPoolSource	= null;
	private SOSVfsConnectionPool							objConnPoolTarget	= null;

	/**
	 *
	 */
	public SOSVfsConnectionFactory(final SOSFTPOptions pobjOptions) {
		super();
		objOptions = pobjOptions;
		createConnectionPool();
	}

	public SOSVfsConnectionPool getSourcePool() {
		return objConnPoolSource;
	}

	public SOSVfsConnectionPool getTargetPool() {
		return objConnPoolTarget;
	}

	public void createConnectionPool() {
		if (objConnPoolSource != null && objOptions.PollingServer.isTrue()) {
			// dann nehmen wir genau diesen Pool
			logger.debug("use existing connection pool");
		}
		else {
			int intMaxParallelTransfers = 1;
			if (objOptions.ConcurrentTransfer.isTrue()) {
				intMaxParallelTransfers = objOptions.MaxConcurrentTransfers.value() + 1;
			}
			if (objConnPoolSource == null || objOptions.reuseConnection.isFalse()) {
				logger.debug("create connection pool");
				objConnPoolSource = new SOSVfsConnectionPool();
				objConnPoolTarget = new SOSVfsConnectionPool();
				for (int i = 0; i < intMaxParallelTransfers; i++) {
					objConnPoolSource.add(getVfsHandler(true));
					// TODO handle multiple targets
					if (objOptions.NeedTargetClient() == true) {
						objConnPoolTarget.add(getVfsHandler(false));
					}
				}
			}
			else {
				// TODO check for changes in the connection options (host, user)
				logger.debug("use existing connection pool");
			}
		}
	}

	private ISOSVFSHandler getVfsHandler(final boolean pflgIsDatasource) {
		ISOSVFSHandler objVFS4Handler = null;
		try {
			SOSConnection2OptionsAlternate objConnectOptions;
			String strDataType;
			if (pflgIsDatasource) {
				objConnectOptions = objOptions.getConnectionOptions().Source();
				strDataType = objOptions.getDataSourceType();
			}
			else {
				objConnectOptions = objOptions.getConnectionOptions().Target();
				strDataType = objOptions.getDataTargetType();
			}
			objConnectOptions.loadClassName.SetIfNotDirty(objOptions.getConnectionOptions().loadClassName);
			VFSFactory.setConnectionOptions(objConnectOptions);
			objVFS4Handler = VFSFactory.getHandler(strDataType);
			objVFS4Handler.Options(objOptions);
			if (pflgIsDatasource) {
				objVFS4Handler.setSource();
				doConnect(objVFS4Handler, objConnectOptions);
				doAuthenticate(objVFS4Handler, objConnectOptions, pflgIsDatasource);
			}
			else {
				objVFS4Handler.setTarget();
				// TODO implement lazyconnectionmode
				doConnect(objVFS4Handler, objConnectOptions);
				String strAuthMethod = objConnectOptions.getAuth_method().Value();
				doAuthenticate(objVFS4Handler, objConnectOptions, pflgIsDatasource);
			}
		}
		catch (Exception ex) {
			throw (RuntimeException) ex;
		}
		return objVFS4Handler;
	}

	private void doConnect(final ISOSVFSHandler objVFS4Handler, final SOSConnection2OptionsAlternate objConnectOptions) {
		try {
			objVFS4Handler.Connect(objConnectOptions);
		}
		catch (Exception e) { // Problem to connect, try alternate host
			// TODO respect alternate data-source type? alternate port etc. ?
			JobSchedulerException.LastErrorMessage = "";
			try {
				objVFS4Handler.Connect(objConnectOptions.Alternatives());
				objConnectOptions.setAlternateOptionsUsed("true");
			}
			catch (Exception e1) {
				throw new JobSchedulerException(e);
			}
			// TODO get an instance of .Alternatives for Authentication ...
		}
	}

	private void doAuthenticate(final ISOSVFSHandler objVFS4Handler, final SOSConnection2OptionsAlternate objConnectOptions, final boolean pflgIsDataSource)
			throws Exception {
		try {
			objVFS4Handler.Authenticate(objConnectOptions);
		}
		catch (Exception e) { // SOSFTP-113: Problem to login, try alternate User
			// TODO respect alternate authentication, eg password and/or public key
			JobSchedulerException.LastErrorMessage = "";
			try {
				objVFS4Handler.Authenticate(objConnectOptions.Alternatives());
			}
			catch (RuntimeException e1) {
				throw e1;
			}
			objConnectOptions.setAlternateOptionsUsed("true");
		}
		ISOSVfsFileTransfer objDataClient = (ISOSVfsFileTransfer) objVFS4Handler;
		if (objOptions.passive_mode.value() || objConnectOptions.passive_mode.isTrue()) {
			objDataClient.passive();
		}
		objDataClient.TransferMode(objOptions.transfer_mode);
		objDataClient.ControlEncoding(objOptions.ControlEncoding.Value());
		// TODO pre-commands for source and target seperately
		if (objOptions.PreFtpCommands.IsNotEmpty() && pflgIsDataSource == false) {
			// TODO Command separator as option
			for (String strCmd : objOptions.PreFtpCommands.split()) {
				strCmd = objOptions.replaceVars(strCmd);
				objDataClient.getHandler().ExecuteCommand(strCmd);
			}
		}
	}

	public void clear() {
		objConnPoolSource.clear();
		objConnPoolTarget.clear();
	}
}
