/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
/**
 *
 */
package com.sos.VirtualFileSystem.DataElements;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.apache.log4j.Logger;

import com.sos.VirtualFileSystem.Interfaces.ISOSVFSHandler;

/**
 * @author KB
 *
 */
public class SOSVfsConnectionPool {
	@SuppressWarnings("unused")
	private final String		conClassName	= this.getClass().getSimpleName();
	@SuppressWarnings("unused")
	private static final String	conSVNVersion	= "$Id: SOSVfsConnectionPool.java 22832 2014-03-11 17:30:56Z kb $";
	@SuppressWarnings("unused")
	private final static Logger		logger		= Logger.getLogger(SOSVfsConnectionPool.class);
	List<ISOSVFSHandler>		list			= Collections.synchronizedList(new ArrayList<ISOSVFSHandler>());

	/**
	 *
	 */
	public SOSVfsConnectionPool() {
	}

	public void add(final ISOSVFSHandler pobjHandler) {
		// TODO avoid duplicate handler
		list.add(pobjHandler);
	}

	public void clear() {
		this.logout();
		list = Collections.synchronizedList(new ArrayList<ISOSVFSHandler>());
	}

	/**
	 *
	*
	* \brief getList
	*
	* \details
	* Returns the list of the current instances of connections
	*
	* \return List<ISOSVFSHandler>
	*
	 */
	public List<ISOSVFSHandler> getList() {
		if (list == null) {
			this.clear();
		}
		return list;
	}

	/**
	 *
	* \brief getUnused
	*
	* \details
	* returns the next unused (idle) connection from the pool.
	* Checks the connection status and (re)connect, if needed.
	* Checks the credentials and make a (re)authentication, if needed.
	* Set a lock to avoid duplicate usage.
	*
	* \return ISOSVFSHandler
	*
	 */
	public ISOSVFSHandler getUnused() {
		for (ISOSVFSHandler objHandler : list) {
			if (objHandler.isLocked() == false ) {
				// TODO check connect and user-id
				objHandler.lock();
				return objHandler;
			}
		}
		return null;
	}

	public ISOSVFSHandler logout() {
		if (list != null) {
			for (ISOSVFSHandler objHandler : list) {
				try {
					objHandler.CloseSession();
				}
				catch (Exception e) {
					e.printStackTrace();
				}
				objHandler.setLogin(false);
				objHandler.release();
			}
		}
		return null;
	}
}
