/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.dailyschedule.db;


import java.io.File;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import org.apache.log4j.Logger;

import com.sos.dailyschedule.job.CheckDailyScheduleMain;
import com.sos.dailyschedule.job.CheckDailyScheduleOptions;
import com.sos.dashboard.globals.DashBoardConstants;
import com.sos.scheduler.history.db.SchedulerTaskHistoryDBItem;
import com.sos.scheduler.history.db.SchedulerOrderHistoryDBItem;
import com.sos.scheduler.history.db.SchedulerOrderHistoryDBLayer;
import com.sos.scheduler.history.db.SchedulerTaskHistoryDBLayer;

/**
* \class DaysScheduleAdjustment 
* 
* \brief DaysScheduleAdjustment - 
* 
* \details
*
* \section DaysScheduleAdjustment.java_intro_sec Introduction
*
* \section DaysScheduleAdjustment.java_samples Some Samples
*
* \code
*   .... code goes here ...
* \endcode
*
* <p style="text-align:center">
* <br />---------------------------------------------------------------------------
* <br /> APL/Software GmbH - Berlin
* <br />##### generated by ClaviusXPress (http://www.sos-berlin.com) #########
* <br />---------------------------------------------------------------------------
* </p>
* \author Uwe Risse
* \version 27.09.2011
* \see reference
*
* Created on 27.09.2011 09:52:55
 */

public class DailyScheduleAdjustment {

	@SuppressWarnings("unused")
	private final String	conClassName	= "DaysScheduleAdjustment";
	private DailyScheduleDBLayer dailyScheduleDBLayer;
	private SchedulerTaskHistoryDBLayer schedulerTaskHistoryDBLayer;
	private SchedulerOrderHistoryDBLayer schedulerOrderHistoryDBLayer;
	private String dateFormat = "yyyy-MM-dd'T'HH:mm:ss";
    private String schedulerId;
	private Date from;
	private Date to;
	private int dayOffset;
	private CheckDailyScheduleOptions options=null;
    private static Logger       logger          = Logger.getLogger(DailyScheduleAdjustment.class);


	public DailyScheduleAdjustment(File configurationFile) {
		dailyScheduleDBLayer = new DailyScheduleDBLayer(configurationFile);
		schedulerTaskHistoryDBLayer = new SchedulerTaskHistoryDBLayer(configurationFile); 
		schedulerOrderHistoryDBLayer = new SchedulerOrderHistoryDBLayer(configurationFile); 
		
		//
	}
	
	private void adjustDaysScheduleItem(DailyScheduleDBItem dailyScheduleItem,List <SchedulerTaskHistoryDBItem> schedulerHistoryList) {

	    logger.debug(String.format("%s records in schedulerHistoryList",schedulerHistoryList.size()));
        for (int i = 0; i<schedulerHistoryList.size(); i++) {
        	SchedulerTaskHistoryDBItem schedulerHistoryDBItem = (SchedulerTaskHistoryDBItem) schedulerHistoryList.get(i);
        	if (!schedulerHistoryDBItem .isAssignToDaysScheduler() /*&& !dailyScheduleDBLayer.contains(schedulerHistoryDBItem)*/) {
            	if (dailyScheduleItem.isStandalone() && dailyScheduleItem.isEqual(schedulerHistoryDBItem)){
            	    logger.debug(String.format("... assign %s to %s", schedulerHistoryDBItem.getId(),dailyScheduleItem.getJobName()));
            	    	dailyScheduleItem.setScheduleExecuted(schedulerHistoryDBItem.getStartTime());
            	    	dailyScheduleItem.setSchedulerHistoryId(schedulerHistoryDBItem.getId());
            	    	dailyScheduleItem.setStatus(DashBoardConstants.STATUS_ASSIGNED);
            	    	dailyScheduleItem.setResult(schedulerHistoryDBItem.getExitCode());
            	    	dailyScheduleDBLayer.update(dailyScheduleItem);
            	    	schedulerHistoryDBItem.setAssignToDaysScheduler(true);
            	        break;
             		}
            	}
             }
        logger.debug(String.format("... could not assign %s planned at:%s",dailyScheduleItem.getJobName(),dailyScheduleItem.getSchedulePlannedFormated() ));

	}
	
    private void adjustDaysScheduleOrderItem(DailyScheduleDBItem dailyScheduleItem, List<SchedulerOrderHistoryDBItem> schedulerOrderHistoryList) {

        logger.debug(String.format("%s records in schedulerOrderHistoryList",schedulerOrderHistoryList.size()));

        for (int i = 0; i < schedulerOrderHistoryList.size(); i++) {
            SchedulerOrderHistoryDBItem schedulerOrderHistoryDBItem = (SchedulerOrderHistoryDBItem) schedulerOrderHistoryList.get(i);
               
            if (!schedulerOrderHistoryDBItem.isAssignToDaysScheduler() /*&& !dailyScheduleDBLayer.contains(schedulerOrderHistoryDBItem)*/) {
                if (dailyScheduleItem.isOrderJob() && dailyScheduleItem.isEqual(schedulerOrderHistoryDBItem)) {
                   
                    logger.debug(String.format("... assign %s to %s/%s", schedulerOrderHistoryDBItem.getHistoryId(),dailyScheduleItem.getJobChainNotNull(),dailyScheduleItem.getOrderId() ));

                    dailyScheduleItem.setScheduleExecuted(schedulerOrderHistoryDBItem.getStartTime());
                    dailyScheduleItem.setSchedulerOrderHistoryId(schedulerOrderHistoryDBItem.getHistoryId());
                   
                    dailyScheduleItem.setStatus(DashBoardConstants.STATUS_ASSIGNED);  

                    if (schedulerOrderHistoryDBItem.haveError()) {
                        dailyScheduleItem.setResult(1);
                    }
                    else {
                        dailyScheduleItem.setResult(0);
                    }
                    dailyScheduleDBLayer.update(dailyScheduleItem);
                    schedulerOrderHistoryDBItem.setAssignToDaysScheduler(true);
                    break;
                }
            }
        }


    }
	
	public void adjustWithHistory()  {
	    String lastSchedulerId="***";

		//Read daysScheduler
		
		dailyScheduleDBLayer.setWhereSchedulerId(this.schedulerId);
		dailyScheduleDBLayer.getFilter().setOrderCriteria("schedulerId");
		dailyScheduleDBLayer.setWhereFrom(from);
		dailyScheduleDBLayer.setWhereTo(to);

		System.out.println("Optimized version....");
		List <DailyScheduleDBItem> dailyScheduleList = dailyScheduleDBLayer.getWaitingDailyScheduleList(-1);
	 
		//read scheduler history
		schedulerTaskHistoryDBLayer.getFilter().setLimit(-1);
		schedulerTaskHistoryDBLayer.getFilter().setExecutedFrom(from);
		schedulerTaskHistoryDBLayer.getFilter().setExecutedTo(dailyScheduleDBLayer.getWhereTo());
 		
		//read scheduler Order history
		schedulerOrderHistoryDBLayer.getFilter().setLimit(-1);
		schedulerOrderHistoryDBLayer.getFilter().setExecutedFrom(from);
		schedulerOrderHistoryDBLayer.getFilter().setExecutedTo(dailyScheduleDBLayer.getWhereTo());
 		   
		dailyScheduleDBLayer.beginTransaction();

        List <SchedulerTaskHistoryDBItem> schedulerHistoryList = null;
        List <SchedulerOrderHistoryDBItem> schedulerOrderHistoryList = null;

		for (int i = 0; i<dailyScheduleList.size(); i++) {
        	DailyScheduleDBItem daysScheduleItem = (DailyScheduleDBItem) dailyScheduleList.get(i);
        	String schedulerId = daysScheduleItem.getSchedulerId();

           	
        	if (daysScheduleItem.isStandalone()) {

                //Fr jede SchedulerId neue Listen holen.
                if (schedulerHistoryList == null || !schedulerId.equals(lastSchedulerId)){
                    dailyScheduleDBLayer.commit();
                    dailyScheduleDBLayer.beginTransaction();
                    schedulerTaskHistoryDBLayer.getFilter().setSchedulerId(schedulerId);
                    schedulerHistoryList = schedulerTaskHistoryDBLayer.getUnassignedSchedulerHistoryListFromTo();
                    lastSchedulerId = schedulerId;
                    logger.debug(String.format("... Reading scheduler_id: %s",schedulerId));

                }
        	    adjustDaysScheduleItem(daysScheduleItem, schedulerHistoryList);
          	}else {
                //Fr jede SchedulerId neue Listen holen.
                if (schedulerOrderHistoryList == null || !schedulerId.equals(lastSchedulerId)){
                    dailyScheduleDBLayer.commit();
                    dailyScheduleDBLayer.beginTransaction();
                    schedulerOrderHistoryDBLayer.getFilter().setSchedulerId(schedulerId);
                    schedulerOrderHistoryList = schedulerOrderHistoryDBLayer.getUnassignedSchedulerOrderHistoryListFromTo();
                    logger.debug(String.format("... Reading scheduler_id: %s",schedulerId));
                    lastSchedulerId = schedulerId;
                }
        	   adjustDaysScheduleOrderItem(daysScheduleItem, schedulerOrderHistoryList);
        	}
        }
    	dailyScheduleDBLayer.commit();

	}
	
	private void setFrom() throws ParseException {
		Date now = new Date();
		if (dayOffset < 0) {
		   GregorianCalendar calendar = new GregorianCalendar();
		   calendar.setTime(now);
		   calendar.add(GregorianCalendar.DAY_OF_MONTH, dayOffset);
		   now = calendar.getTime();
		}

		SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        String froms = formatter.format(now);
    	froms = froms + "T00:00:00";
		formatter = new SimpleDateFormat(dateFormat);
		this.from = formatter.parse(froms);	
	}
	
	private void setTo() throws ParseException {
		Date now = new Date();
		if (dayOffset > 0) {
		   GregorianCalendar calendar = new GregorianCalendar();
		   calendar.setTime(now);
		   calendar.add(GregorianCalendar.DAY_OF_MONTH, dayOffset);
		   now = calendar.getTime();
		}

		SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        String tos = formatter.format(now);
    	tos = tos + "T23:59:59";
		formatter = new SimpleDateFormat(dateFormat);
		this.to = formatter.parse(tos);
	}
	
	public Date getFrom() {
		return from;
	}

	public Date getTo() {
		return to;
	}

	public void setOptions(CheckDailyScheduleOptions options) throws ParseException {
		this.options = options;
		schedulerId = this.options.getscheduler_id().Value();
		dayOffset = this.options.getdayOffset().value();
		setFrom();
		setTo();
	}

    public DailyScheduleDBLayer getDailyScheduleDBLayer() {
        return dailyScheduleDBLayer;
    }

    public void setDailyScheduleDBLayer(DailyScheduleDBLayer dailyScheduleDBLayer) {
        this.dailyScheduleDBLayer = dailyScheduleDBLayer;
    }

    public void setTo(Date to) {
        this.to = to;
    }

}
