/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.graphviz;

import com.sos.graphviz.enums.*;
import com.sos.graphviz.properties.GraphvizEnumProperty;
import com.sos.graphviz.properties.GraphvizProperty;

public class SubgraphProperties extends GraphvizObject implements IGraphvizObject {

    private static final String constProlog = "";
	private static final String constEpilog = "";

    private GraphvizProperty fontColor = new GraphvizEnumProperty("fontcolor");
    private GraphvizProperty fontName = new GraphvizProperty("fontname");
    private GraphvizProperty fontSize = new GraphvizProperty("fontsize");
    private GraphvizProperty id = new GraphvizProperty("id");
	private GraphvizProperty rank = new GraphvizEnumProperty("rank");
	private GraphvizProperty style = new GraphvizEnumProperty("style");
    private GraphvizProperty label = new GraphvizProperty("label");
    private GraphvizProperty labeljust = new GraphvizEnumProperty("labeljust");
    private GraphvizProperty ordering = new GraphvizEnumProperty("ordering");
	private GraphvizProperty bgcolor = new GraphvizEnumProperty("bgcolor");
	private GraphvizProperty color = new GraphvizEnumProperty("color");

	public SubgraphProperties(RankType rankType) {
		super(constProlog, constEpilog);
		this.rank.setValue(rankType);
	}

	public SubgraphProperties() {
		super(constProlog, constEpilog);
	}

    @Override
	public String getContent() {
		StringBuilder sb = new StringBuilder();
        sb.append( bgcolor.getContent() );
        sb.append( color.getContent() );
        sb.append( fontColor.getContent() );
        sb.append( fontName.getContent() );
        sb.append( fontSize.getContent() );
        sb.append( id.getContent() );
		sb.append( label.getContent() );
		sb.append( labeljust.getContent() );
        sb.append(rank.getContent());
        sb.append( style.getContent() );
		return sb.toString();
	}

	public Style getStyle() {
		return (Style)style.getValue();
	}

	public void setStyle(Style style) {
		this.style.setValue(style);
	}

	public GraphvizObject getProperties() {
		return this;
	}

	public String getLabel() {
		return (String)label.getValue();
	}

	public void setLabel(String label) {
		this.label.setValue(label);
	}

	public SVGColor getBgcolor() {
		return (SVGColor)bgcolor.getValue();
	}

	public void setBgcolor(SVGColor bgcolor) {
		this.bgcolor.setValue(bgcolor);
	}

	public SVGColor getColor() {
		return (SVGColor)color.getValue();
	}

	public void setColor(SVGColor color) {
		this.color.setValue(color);
	}

	public void setRank(RankType rankType) {
		this.rank.setValue(rankType);
	}

	public RankType getRank() {
		return (RankType)this.rank.getValue();
	}

    public Justify getLabeljust() {
        return (Justify)labeljust.getValue();
    }

    public void setLabeljust(Justify labeljust) {
        this.labeljust.setValue(labeljust);
    }

    public Ordering getOrdering() {
        return (Ordering)ordering.getValue();
    }

    public void setOrdering(Ordering ordering) {
        this.ordering.setValue(ordering);
    }

    public SVGColor getFontColor() {
        return (SVGColor)fontColor.getValue();
    }

    public String getFontName() {
        return (String)fontName.getValue();
    }

    public double getFontSize() {
        return (Double)fontSize.getValue();
    }

    public void setFontColor(SVGColor fontColor) {
        this.fontColor.setValue(fontColor);
    }

    public void setFontName(String fontName) {
        this.fontName.setValue(fontName);
    }

    public void setFontSize(double fontSize) {
        this.fontSize.setValue(fontSize);
    }

    public String getId() {
        return (String)id.getValue();
    }

    public void setId(String id) {
        this.id.setValue(id);
    }

}
