/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.jitl.eventing.checkevents;
import java.io.File;

import org.apache.log4j.Logger;

import com.sos.JSHelper.Basics.JSJobUtilities;
import com.sos.JSHelper.Basics.JSJobUtilitiesClass;
import com.sos.eventing.db.SchedulerEventDBLayer;
import com.sos.eventing.db.SchedulerEventFilter;

/**
 * \class 		JobSchedulerCheckEvents - Workerclass for "Check if events exist"
 *
 * \brief AdapterClass of JobSchedulerCheckEvents for the SOSJobScheduler
 *
 * This Class JobSchedulerCheckEvents is the worker-class.
 *

 *
 *
 * \verbatim ;
 * \endverbatim
 */
public class JobSchedulerCheckEvents extends JSJobUtilitiesClass<JobSchedulerCheckEventsOptions> implements JSJobUtilities {
	@SuppressWarnings("unused") private final String conClassName = this.getClass().getSimpleName();
	@SuppressWarnings("unused") private static final String conSVNVersion = "$Id$";
	@SuppressWarnings("unused") private final Logger logger = Logger.getLogger(this.getClass());
	

//	protected JobSchedulerCheckEventsOptions	objOptions			= null;
//	private JSJobUtilities						objJSJobUtilities	= this;
	protected boolean							exist				= false;
  
	
	/**
	 * 
	 * \brief JobSchedulerCheckEvents
	 *
	 * \details
	 *
	 */
	public JobSchedulerCheckEvents() {
		super();
	}

	/**
	 * 
	 * \brief Options - returns the JobSchedulerCheckEventsOptionClass
	 * 
	 * \details
	 * The JobSchedulerCheckEventsOptionClass is used as a Container for all Options (Settings) which are
	 * needed.
	 *  
	 * \return JobSchedulerCheckEventsOptions
	 *
	 */
	@Override
	public JobSchedulerCheckEventsOptions Options() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::Options"; //$NON-NLS-1$
		if (objOptions == null) {
			objOptions = new JobSchedulerCheckEventsOptions();
		}
		return objOptions;
	}

	/**
	 * 
	 * \brief Execute - Start the Execution of JobSchedulerCheckEvents
	 * 
	 * \details
	 * 
	 * For more details see
	 * 
	 * \see JobSchedulerAdapterClass 
	 * \see JobSchedulerCheckEventsMain
	 * 
	 * \return JobSchedulerCheckEvents
	 *
	 * @return
	 */
	public JobSchedulerCheckEvents Execute() throws Exception {
		final String conMethodName = conClassName + "::Execute"; //$NON-NLS-1$
		try {
			Options().CheckMandatory();
			logger.debug(Options().toString());
			exist = false;
			SchedulerEventDBLayer schedulerEventDBLayer = new SchedulerEventDBLayer(new File(objOptions.configuration_file.Value()));
			if (objOptions.event_condition.isDirty()) {
				if (objOptions.event_class.isDirty()) {
					exist = schedulerEventDBLayer.checkEventExists(objOptions.event_condition.Value(), objOptions.event_class.Value());
				}
				else {
					exist = schedulerEventDBLayer.checkEventExists(objOptions.event_condition.Value());
				}
			}
			else {
				SchedulerEventFilter schedulerEventFilter = new SchedulerEventFilter();
				schedulerEventFilter.setEventClass(objOptions.event_class.Value());
				schedulerEventFilter.setEventId(objOptions.event_id.Value());
				schedulerEventFilter.setExitCode(objOptions.event_exit_code.Value());
				schedulerEventFilter.setSchedulerId(objOptions.event_scheduler_id.Value());
				schedulerEventFilter.setRemoteSchedulerHost(objOptions.remote_scheduler_host.Value());
				schedulerEventFilter.setRemoteSchedulerPort(objOptions.remote_scheduler_port.Value());
				schedulerEventFilter.setJobChain(objOptions.event_job_chain.Value());
				schedulerEventFilter.setOrderId(objOptions.event_order_id.Value());
				schedulerEventFilter.setJobName(objOptions.event_job.Value());
				exist = schedulerEventDBLayer.checkEventExists(schedulerEventFilter);
			}
		}
		catch (Exception e) {
			e.printStackTrace(System.err);
			throw e;
		}
		finally {
		}
		return this;
	}

	public void init() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::init"; //$NON-NLS-1$
		doInitialize();
	}

	private void doInitialize() {
	} // doInitialize

	@Override
	public String myReplaceAll(final String pstrSourceString, final String pstrReplaceWhat, final String pstrReplaceWith) {
		String newReplacement = pstrReplaceWith.replaceAll("\\$", "\\\\\\$");
		return pstrSourceString.replaceAll("(?m)" + pstrReplaceWhat, newReplacement);
	}

	/**
	 * 
	 * \brief replaceSchedulerVars
	 * 
	 * \details
	 * Dummy-Method to make sure, that there is always a valid Instance for the JSJobUtilities.
	 * \return 
	 *
	 * @param isWindows
	 * @param pstrString2Modify
	 * @return
	 */
	@Override public String replaceSchedulerVars(final boolean isWindows, final String pstrString2Modify) {
		logger.debug("replaceSchedulerVars as Dummy-call executed. No Instance of JobUtilites specified.");
		return pstrString2Modify;
	}

	/**
	 * 
	 * \brief setJSParam
	 * 
	 * \details
	 * Dummy-Method to make shure, that there is always a valid Instance for the JSJobUtilities.
	 * \return 
	 *
	 * @param pstrKey
	 * @param pstrValue
	 */
	@Override public void setJSParam(final String pstrKey, final String pstrValue) {
	}

	@Override public void setJSParam(final String pstrKey, final StringBuffer pstrValue) {
	}

	/**
	 * 
	 * \brief setJSJobUtilites
	 * 
	 * \details
	 * The JobUtilities are a set of methods used by the SSH-Job or can be used be other, similar, job-
	 * implementations.
	 * 
	 * \return void
	 *
	 * @param pobjJSJobUtilities
	 */
	@Override
	public void setJSJobUtilites(final JSJobUtilities pobjJSJobUtilities) {
		if (pobjJSJobUtilities == null) {
			objJSJobUtilities = this;
		}
		else {
			objJSJobUtilities = pobjJSJobUtilities;
		}
		logger.debug("objJSJobUtilities = " + objJSJobUtilities.getClass().getName());
	}

	@Override public String getCurrentNodeName() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override public void setStateText(final String pstrStateText) {
		// TODO Auto-generated method stub
	}

	@Override public void setCC(final int pintCC) {
		// TODO Auto-generated method stub
	}

    @Override
    public void setNextNodeState(String pstrNodeName) {
        // TODO Auto-generated method stub
        
    }
} // class JobSchedulerCheckEvents
