/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.cmd;

import com.sos.JSHelper.Exceptions.JobSchedulerException;
import com.sos.scheduler.model.commands.JSCmdCommands;
import com.sos.scheduler.model.objects.JSObjBase;
import org.apache.log4j.Logger;

import javax.xml.bind.JAXBElement;
import java.util.ArrayList;
import java.util.List;

/**
 * A container class to collect commands and send them all together to an JobScheduler instance.
 */
public class JSCommands extends JSCommand {

    private static Logger logger = Logger.getLogger(JSCommands.class);

    private final List<String> listOfCommands = new ArrayList<String>();
    private final JSCmdCommands cmdContainer;
    private String xmlHeader = "";

    public JSCommands(String host, Integer port) {
        super(host, port);
        this.cmdContainer = getFactory().createCmdCommands();
        setCommand(cmdContainer);
        listOfCommands.clear();
    }

    public void addCommand(JSCommand command) {
        if(xmlHeader.isEmpty()) {
            String xml = command.getCommand().toXMLString();
            xmlHeader = xml.substring(0,xml.indexOf("?>")+2);
        }
        listOfCommands.add(command.getCommand().toXMLString().replace(xmlHeader,""));
    }

    @Override
    public void run() {

        StringBuffer xmlCmd = new StringBuffer();
        xmlCmd.append(xmlHeader);
        xmlCmd.append("<commands>");
        for(String cmd : listOfCommands) {
            xmlCmd.append(cmd.trim());
        }
        xmlCmd.append("</commands>");
        logger.info(xmlCmd.toString());
        JAXBElement<JSObjBase> obj = (JAXBElement<JSObjBase>) cmdContainer.unMarshal(xmlCmd.toString());
        cmdContainer.setObjectFieldsFrom(obj.getValue());
        logger.info(cmdContainer.toXMLString());

        try{
            super.run();
        } catch (Exception e) {
            String msg = "Error fetching commands object.";
            throw new JobSchedulerException(msg,e);
        }

    }

}
