/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.testframework.h2;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.sql.SQLException;

/**
 * To build a connection to a given database.
 */
public class H2PersistentConnection extends H2Connection {

    private final String dbName;
    private final File workingDirectory;

    private final static Logger logger = LoggerFactory.getLogger(H2PersistentConnection.class);

    public H2PersistentConnection(File dbLocation, String dbName, ResourceList fileList) {
        super( H2ConnectionType.FILE_BASED, fileList );
        this.dbName = dbName;
        this.workingDirectory = dbLocation;
        dbLocation.mkdirs();
    }

    public File getDatabaseLocation() {
        return workingDirectory;
    }

    public void close() {
        if (getConnection() != null ) {
            try {
                this.getConnection().close();
            } catch (SQLException e) {
                throw new RuntimeException("Error closing DB: " + getConnectionString(),e);
            }
        }
        super.close();
    }

    public String getDatabaseName() {
        return dbName;
    }

    @Override
    final protected String getConnectionURL() {
        StringBuffer result = new StringBuffer();
        result.append(super.getConnectionURL());
        result.append(getDatabaseLocation());
        result.append("/");
        result.append(dbName);
        return result.toString();
    }

}
