/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
//
// $Id: folder.cxx 14329 2011-05-12 13:43:35Z ss $        Joacim Zschimmer, Zschimmer GmbH, http://www.zschimmer.com

#include "spooler.h"

namespace sos {
namespace scheduler {
namespace folder {



//  VERSUCH EINER KLRUNG            Joacim Zschimmer 2008-05-01
//
//  Wie das Modell der File_based, Dependant, Requisiten, Subsysteme usw. aussehen sollte, 
//  schlt sich erst nach und nach heraus.
//
//
//
//  GENERISCHE ZUSTNDE
//
//  Auer den File_based gibt es noch andere Objekte, die die generischen Zustnde von File_based haben sollten,
//  z.B. Job_chain_node und Schedule_use.
//
//  Ein Objekt mit generischem Zustand sollte ein In_generic_state sein. Dependant sollte ein In_generic_state sein.
//  initialized(), load(), activate() gehren hierher.
//
//  s_undefined,            // Anfnglicher Zustand
//  s_not_initialized,      // set_dom() noch nicht aufgerufen (oder war fehlerhaft)
//  s_initialized,          // on_initialized() aufgerufen. Replacement kann jetzt das alte Objekt ersetzen
//  s_loaded,               // Mit Daten gefllt: bei Jobs die Task-Warteschlange, bei Jobketten die Auftragswarteschlangen. Viele Objekte brauchen das nicht.
//  s_incomplete,           // Requisite fehlt (beim Versuch zu aktivieren)
//  s_active,               // kann wieder zu s_incomplete wechseln
//  s_closed                // Hier gibt's kein Zurck. Es sollte gleich darauf zerstrt werden.
//
//  Zu jedem file_based<> gibt es ein typed_folder<> und ein file_based_subsystem<>.
//
//  Fr Dependant und Schedule_use brauchen wir kein typed_folder() und statt einem Subsystem 
//  wrde ein Object_descriptor oder so etwas einfacheres gengen. Der lieferte den XML-Namen, dom_element() usw.
//
//  Vielleicht kann jedes Scheduler_object in einem generischen Zustand sein?
//
//
//
//  SPEZIELLE ZUSTNDE (Nebenzustnde)
//
//  Manche Objekte brauchen weitere Zustnde:
//
//  Job:        s_stopping, s_stopped, s_error,
//              s_pending und s_running ist vielleicht unerheblich und knnten wie das generische s_active sein.
//  Job_chain:  s_stopped
//  Scheduler:  s_stopped, s_loading, s_starting, s_waiting_for_activation, s_paused, s_stopping, s_stopping_let_run
//  Task:       zahlose Zustnde
//
//  s_paused (s_halted, s_stopped) knnte verallgemeinert werden. 
//  Eigentlich kein eigener Zustand, sondern eine Eigenschaft parallel zu den generischen Zustand
//  (auch ein nicht initialisiertes Objekt knnte angehalten werden. Lediglich die Aktvierung soll verhindert werden).
//
//
//
//  REQUISITEN
//
//  Dependants knnen Requisiten (Requisite_path) haben.
//  Ein Requisit wird von einem Dependant zum Betrieb (s_active) gebraucht.
//  Wenn ein Requisit fehlt, sollte der Zustand s_incomplete statt s_active sein (das ist neu und nicht fr alle realisiert).
//  
//  Wenn ein Requisite geladen ist (s_loaded), kann ein Dependant es benutzen: on_requisite_loaded().
//  Die Standard-Implementierung sollte den Dependant aktivieren (activate()). Das ist jetzt leider nicht mglich, weil Dependant die Zustnde nicht kennt.
//
//  add_requisite() sollte vielleicht erst von on_load() gerufen werden.
//  Nur Objekte im Zustand s_loaded oder weiter sollten Requisiten haben.
//
//  Es sollte registriert sein, ob ein Objekt alle Requisiten hat.
//  Wenn eine fehlt, sollte das Objekt im Zustand s_incomplete und nicht aktivierbar sein.
//  Sobald alle Requisiten da sind, sollte das Objekt automatisch aktiviert werden.
//
//
//  Eine Requisite fehlt nicht, wenn sie mindestens im Zustand s_loaded ist.
//  Damit wren aber indirekte Abhngigkeiten nicht erkennbar (denn eine Requisite in s_incomplete gilt jetzt als vorhanden).
//  Das jetzt vielleicht nicht ganz sauber codiert und knnte geklrt, vielleicht verallgemeinert werden.
//
//  Oder so: Der Scheduler knnte die Aktivierungen so ordnen, dass erst die Requisiten dann die Depandents aktiviert werden.
//  Wir brauchen also die transitive Hlle.
//
//
//
//  USE
//
//  Wenn ein Objekt (Dependant) ein anderes Objekt (die Requisite) braucht, kann dazwischen ein Gebrauchobjekt sein, der Use.
//  Z.B. lock::Use und Schedule_use. 
//
//  Mglichweise lsst sich das verallgemeinern:
//      Job_chain_node ist ein Use
//
//  Dependant hat mehrere Requisiten
//  Use hat genau eine Requisite
//  Also: Dependant hat mehrere Use, haben je eine Requiste. Oder so. Noch unklar.
//
//
//
//  INLAY
//
//  Damit Objekte austauschbar sind, ohne das Dependant davon gro Notiz nehmen mssen (also den Verweis erneuern),
//  knnen Objekte ein austauschbares Inlay habe. 
//  Bei nderung des Objekts bleibt das Objekt erhalten, aber das Inlay wird ausgetauscht: Schedule::Inlay.
//
//  Das knnte ein allgemeines Verfahren fr das Ersetzen eines Objekts sein.
//      replace_with()
//      prepare_replacement(),
//      on_replace_now(): Tauscht nur Inlay aus, statt bisher remove_file_based(), add_file_based().
//
//  set_inlay() verallgemeinern?
//
//  Vielleicht ein neuer Aufruf: on_requisite_replaced()
//
//
//
//  FEHLER-ZUSTNDE
//
//  _base_file_xc ist fr Fehler beim Laden der Datei (inkl. set_dom()).
//  Die Objekte haben noch ihre eigenen Fehler, was verallgemeinert werden knnte.
//  Ein Objekt kann immer nur einen Fehler haben?
//  Ein fehlerhaftes Objekt kann nicht aktiv sein. Was ist es dann? Neuer Zustand s_error?
//  Was sind das berhaupt fr Fehler? Unerwartete Exception? Nur bei Job?
//  Wenn kein Fehler vorliegt, aber eine Requisite fehlt, liefert die Fehler-Methde die fehlenden Requisiten.
//
//  Ein XML-Kommando knnte die fehlerhaften, unvollstndigen, nicht aktiven Objekte liefern.
//
//
//
//  HTML-OBERFLCHE
//
//  Wenn das alles so hbsch verallgemeinert sein sollte,
//  kann auch die HTML-Oberflche verallgemeinert werden:
//  Zu jedem Objekt knnten die fehlenden Requisiten, Fehler, generischer Zustand usw. gezeigt werden.
//
//

//--------------------------------------------------------------------------------------------const

const char                      folder_separator            = '/';
const char                      folder_name_separator       = ',';                                  // Fr mehrteilige Dateinamen: "jobchain,orderid.order.xml"

//-------------------------------------------------------------------------------------------------

} //namespace folder
} //namespace scheduler
} //namespace sos
